@@ -5,26 +5,132 @@ import (
 	"strings"
 )
 
+// This test demonstrates that the shell tokenizer is not perfect yet.
+// There are still some corner cases that trigger a parse error.
+// To get 100% code coverage, they have been found using the fuzzer
+// and trimmed down to minimal examples.
+func (s *Suite) Test_ShTokenizer__examples_from_fuzzing(c *check.C) {
+	t := s.Init(c)
+
+	test := func(input string, diagnostics ...string) {
+		mklines := t.NewMkLines("filename.mk",
+			MkCvsID,
+			"\t"+input)
+		mklines.Check()
+		t.CheckOutput(diagnostics)
+	}
+
+	// Covers shAtomBacktDquot: return nil.
+	// These are nested backticks with double quotes,
+	// which should be avoided since POSIX marks them as unspecified.
+	test(
+		"`\"`",
+		"WARN: filename.mk:2: Internal pkglint error in ShTokenizer.ShAtom at \"`\" (quoting=bd).",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"`\\\"`\"")
+
+	// Covers shAtomBacktSquot: return nil
+	test(
+		"`'$`",
+		"WARN: filename.mk:2: Internal pkglint error in ShTokenizer.ShAtom at \"$`\" (quoting=bs).",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"`'$`\"",
+		"WARN: filename.mk:2: Internal pkglint error in MkLine.Tokenize at \"$`\".")
+
+	// Covers shAtomDquotBacktSquot: return nil
+	test(
+		"\"`'`y",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`'`y\"")
+
+	// Covers shAtomDquotBackt: return nil
+	// XXX: Pkglint should parse unescaped dollar in the same way, everywhere.
+	test(
+		"\"`$|",
+		"WARN: filename.mk:2: Internal pkglint error in ShTokenizer.ShAtom at \"$|\" (quoting=db).",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`$|\"",
+		"WARN: filename.mk:2: Internal pkglint error in MkLine.Tokenize at \"$|\".")
+
+	// Covers shAtomDquotBacktDquot: return nil
+	// XXX: Pkglint should support unlimited nesting.
+	test(
+		"\"`\"`",
+		"WARN: filename.mk:2: Internal pkglint error in ShTokenizer.ShAtom at \"`\" (quoting=dbd).",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`\\\"`\"")
+
+	// Covers shAtomSubshDquot: return nil
+	test(
+		"$$(\"'",
+		"WARN: filename.mk:2: Invoking subshells via $(...) is not portable enough.")
+
+	// Covers shAtomSubsh: case lexer.AdvanceStr("`")
+	test(
+		"$$(`",
+		"WARN: filename.mk:2: Invoking subshells via $(...) is not portable enough.")
+
+	// Covers shAtomSubshSquot: return nil
+	test(
+		"$$('$)",
+		"WARN: filename.mk:2: Internal pkglint error in ShTokenizer.ShAtom at \"$)\" (quoting=Ss).",
+		"WARN: filename.mk:2: Invoking subshells via $(...) is not portable enough.",
+		"WARN: filename.mk:2: Internal pkglint error in MkLine.Tokenize at \"$)\".")
+
+	// Covers shAtomDquotBackt: case lexer.AdvanceRegexp("^#[^`]*")
+	test(
+		"\"`# comment",
+		"WARN: filename.mk:2: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`# comment\"")
+}
+
+// In order to get 100% code coverage for the shell tokenizer, a panic() statement has been
+// added to each uncovered basic block. After that, this fuzzer quickly found relatively
+// small example programs that led to the uncovered code.
+//
+// This test is not useful as-is.
+func (s *Suite) Test_ShTokenizer__fuzzing(c *check.C) {
+	t := s.Init(c)
+
+	fuzzer := NewFuzzer()
+	fuzzer.Char("\"'`$();|_#", 10)
+	fuzzer.Range('a', 'z', 5)
+
+	// This "real" line is necessary because the autofix
+	// in MkParser.varUseBrace checks this.
+	line := t.NewLine("Makefile", 1, "\t:")
+
+	defer fuzzer.CheckOk()
+	for i := 0; i < 1000; i++ {
+		tokenizer := NewShTokenizer(line, fuzzer.Generate(50))
+		tokenizer.ShAtoms()
+		t.Output() // Discard the output, only react on panics.
+	}
+	fuzzer.Ok()
+}
+
 func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 	t := s.Init(c)
 
 	// testRest ensures that the given string is parsed to the expected
 	// atoms, and returns the remaining text.
-	testRest := func(s string, expectedAtoms ...*ShAtom) string {
-		p := NewShTokenizer(dummyLine, s, false)
-		q := shqPlain
-		for _, expectedAtom := range expectedAtoms {
-			c.Check(p.ShAtom(q), deepEquals, expectedAtom)
-			q = expectedAtom.Quoting
+	testRest := func(s string, expectedAtoms []*ShAtom, expectedRest string) {
+		line := t.NewLine("filename.mk", 1, "")
+		p := NewShTokenizer(line, s)
+
+		actualAtoms := p.ShAtoms()
+
+		t.CheckEquals(p.Rest(), expectedRest)
+		t.CheckEquals(len(actualAtoms), len(expectedAtoms))
+
+		for i, actualAtom := range actualAtoms {
+			if i < len(expectedAtoms) {
+				t.CheckDeepEquals(actualAtom, expectedAtoms[i])
+			} else {
+				t.CheckDeepEquals(actualAtom, nil)
+			}
 		}
-		return p.Rest()
 	}
+	atoms := func(atoms ...*ShAtom) []*ShAtom { return atoms }
 
 	// test ensures that the given string is parsed to the expected
 	// atoms, and that the text is completely consumed by the parser.
 	test := func(str string, expected ...*ShAtom) {
-		rest := testRest(str, expected...)
-		c.Check(rest, equals, "")
+		testRest(str, expected, "")
 		t.CheckOutputEmpty()
 	}
 
@@ -34,13 +140,15 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 
 	operator := func(s string) *ShAtom { return atom(shtOperator, s) }
 	comment := func(s string) *ShAtom { return atom(shtComment, s) }
-	mkvar := func(varname string, modifiers ...string) *ShAtom {
+	mkvar := func(varname string, modifiers ...MkVarUseModifier) *ShAtom {
 		text := "${" + varname
 		for _, modifier := range modifiers {
-			text += ":" + strings.Replace(strings.Replace(modifier, "\\", "\\\\", -1), ":", "\\:", -1)
+			escapedBackslash := strings.Replace(modifier.String(), "\\", "\\\\", -1)
+			escapedColon := strings.Replace(escapedBackslash, ":", "\\:", -1)
+			text += ":" + escapedColon // TODO: modifier.Quoted
 		}
 		text += "}"
-		varuse := NewMkVarUse(varname, modifiers...)
+		varuse := NewMkTokenBuilder().VarUse(varname, modifiers...)
 		return &ShAtom{shtVaruse, text, shqPlain, varuse}
 	}
 	shvar := func(text, varname string) *ShAtom { return &ShAtom{shtShVarUse, text, shqPlain, varname} }
@@ -52,31 +160,35 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 	pipe := operator("|")
 	subshell := atom(shtSubshell, "$$(")
 
-	q := func(q ShQuoting, atom *ShAtom) *ShAtom {
-		return &ShAtom{atom.Type, atom.MkText, q, atom.data}
+	q := func(q ShQuoting) func(atom *ShAtom) *ShAtom {
+		return func(atom *ShAtom) *ShAtom {
+			return &ShAtom{atom.Type, atom.MkText, q, atom.data}
+		}
 	}
-	backt := func(atom *ShAtom) *ShAtom { return q(shqBackt, atom) }
-	dquot := func(atom *ShAtom) *ShAtom { return q(shqDquot, atom) }
-	squot := func(atom *ShAtom) *ShAtom { return q(shqSquot, atom) }
-	subsh := func(atom *ShAtom) *ShAtom { return q(shqSubsh, atom) }
-	backtDquot := func(atom *ShAtom) *ShAtom { return q(shqBacktDquot, atom) }
-	backtSquot := func(atom *ShAtom) *ShAtom { return q(shqBacktSquot, atom) }
-	dquotBackt := func(atom *ShAtom) *ShAtom { return q(shqDquotBackt, atom) }
-	subshDquot := func(atom *ShAtom) *ShAtom { return q(shqSubshDquot, atom) }
-	subshSquot := func(atom *ShAtom) *ShAtom { return q(shqSubshSquot, atom) }
-	dquotBacktDquot := func(atom *ShAtom) *ShAtom { return q(shqDquotBacktDquot, atom) }
-	dquotBacktSquot := func(atom *ShAtom) *ShAtom { return q(shqDquotBacktSquot, atom) }
+	backt := q(shqBackt)
+	dquot := q(shqDquot)
+	squot := q(shqSquot)
+	subsh := q(shqSubsh)
+	backtDquot := q(shqBacktDquot)
+	backtSquot := q(shqBacktSquot)
+	dquotBackt := q(shqDquotBackt)
+	subshDquot := q(shqSubshDquot)
+	subshSquot := q(shqSubshSquot)
+	subshBackt := q(shqSubshBackt)
+	dquotBacktDquot := q(shqDquotBacktDquot)
+	dquotBacktSquot := q(shqDquotBacktSquot)
 
 	// Ignore unused functions; useful for deleting some of the tests during debugging.
 	use := func(args ...interface{}) {}
-	use(testRest, test)
+	use(testRest, test, atoms)
 	use(operator, comment, mkvar, text, whitespace)
-	use(space, semicolon, pipe, subshell)
+	use(space, semicolon, pipe, subshell, shvar)
 	use(backt, dquot, squot, subsh)
 	use(backtDquot, backtSquot, dquotBackt, subshDquot, subshSquot)
 	use(dquotBacktDquot, dquotBacktSquot)
 
-	test("" /* none */)
+	test("",
+		nil...)
 
 	test("$$var",
 		shvar("$$var", "var"))
@@ -94,8 +206,8 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 		squot(text("single-quoted")),
 		text("'"))
 
-	rest := testRest("\"" /* none */)
-	c.Check(rest, equals, "\"")
+	test("\"",
+		dquot(text("\"")))
 
 	test("$${file%.c}.o",
 		shvar("$${file%.c}", "file"),
@@ -279,15 +391,16 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 		semicolon,
 		shvar("$$-", "-"))
 
-	rest = testRest("COMMENT=\t\\Make $$$$ fast\"",
+	test("COMMENT=\t\\Make $$$$ fast\"",
+
 		text("COMMENT="),
 		whitespace("\t"),
 		text("\\Make"),
 		space,
 		shvar("$$$$", "$"),
 		space,
-		text("fast"))
-	c.Check(rest, equals, "\"")
+		text("fast"),
+		dquot(text("\"")))
 
 	test("var=`echo;echo|echo&echo||echo&&echo>echo`",
 		text("var="),
@@ -359,7 +472,7 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 	test("$$(cat)",
 		subsh(subshell),
 		subsh(text("cat")),
-		text(")"))
+		operator(")"))
 
 	test("$$(cat 'file')",
 		subsh(subshell),
@@ -368,12 +481,12 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 		subshSquot(text("'")),
 		subshSquot(text("file")),
 		subsh(text("'")),
-		text(")"))
+		operator(")"))
 
 	test("$$(# comment) arg",
 		subsh(subshell),
 		subsh(comment("# comment")),
-		text(")"),
+		operator(")"),
 		space,
 		text("arg"))
 
@@ -388,12 +501,60 @@ func (s *Suite) Test_ShTokenizer_ShAtom(c *check.C) {
 		subshSquot(text("'")),
 		subshSquot(text("second")),
 		subsh(text("'")),
-		text(")"))
+		operator(")"))
+
+	test("$$(echo `echo nested-subshell`)",
+		subsh(subshell),
+		subsh(text("echo")),
+		subsh(space),
+		subshBackt(text("`")),
+		subshBackt(text("echo")),
+		subshBackt(space),
+		subshBackt(text("nested-subshell")),
+		subsh(operator("`")),
+		operator(")"))
+
+	// Subshell with unbalanced parentheses. Many shells (and pkglint)
+	// fail this test, therefore please don't write code like this.
+	//
+	// See NetBSD/src/build.sh, around line 160.
+	test("var=$$(case x in x) still-subshell;; esac);",
+		text("var="), subsh(subshell),
+		subsh(text("case")), subsh(space), subsh(text("x")), subsh(space),
+		subsh(text("in")), subsh(space), subsh(text("x")),
+		// XXX: The parenthesis is for the case pattern, not the end of the subshell.
+		operator(")"), space,
+		text("still-subshell"), operator(";;"), space,
+		text("esac"), operator(")"), operator(";"))
+
+	testRest("`echo \\${VAR}`",
+		atoms(
+			backt(text("`")),
+			backt(text("echo")),
+			backt(space)),
+		"\\${VAR}`")
+	t.CheckOutputLines(
+		"WARN: filename.mk:1: Internal pkglint error " +
+			"in ShTokenizer.ShAtom at \"\\\\${VAR}`\" (quoting=b).")
+
+	// The remaining tokens are shortened in the warning.
+	testRest("`echo \\${VAR.123456789012345678901234567890}`",
+		atoms(
+			backt(text("`")),
+			backt(text("echo")),
+			backt(space)),
+		"\\${VAR.123456789012345678901234567890}`")
+	t.CheckOutputLines(
+		"WARN: filename.mk:1: Internal pkglint error in ShTokenizer.ShAtom " +
+			"at \"\\\\${VAR.1234567890123...\" (quoting=b).")
 }
 
 func (s *Suite) Test_ShTokenizer_ShAtom__quoting(c *check.C) {
+	t := s.Init(c)
+
 	test := func(input, expectedOutput string) {
-		p := NewShTokenizer(dummyLine, input, false)
+		line := t.NewLine("filename.mk", 1, "")
+		p := NewShTokenizer(line, input)
 		q := shqPlain
 		result := ""
 		for {
@@ -407,8 +568,8 @@ func (s *Suite) Test_ShTokenizer_ShAtom__quoting(c *check.C) {
 				result += "[" + q.String() + "]"
 			}
 		}
-		c.Check(result, equals, expectedOutput)
-		c.Check(p.Rest(), equals, "")
+		t.CheckEquals(result, expectedOutput)
+		t.CheckEquals(p.Rest(), "")
 	}
 
 	test("hello, world", "hello, world")
@@ -425,84 +586,30 @@ func (s *Suite) Test_ShTokenizer_ShAtom__quoting(c *check.C) {
 	test("x`x\\\"x\\'x\\`x\\\\", "x`[b]x\\\"x\\'x\\`x\\\\")
 }
 
-func (s *Suite) Test_ShTokenizer_ShToken(c *check.C) {
+// The switch statement in ShTokenizer.ShAtom is exhaustive.
+// If a new quoting mode is added (in which case the shell tokenizer
+// should rather be rewritten completely and correctly), it is ok
+// to panic if ShQuoting is not adjusted in the same commit.
+func (s *Suite) Test_ShTokenizer_ShAtom__internal_error(c *check.C) {
 	t := s.Init(c)
 
-	// testRest ensures that the given string is parsed to the expected
-	// tokens, and returns the remaining text.
-	testRest := func(str string, expected ...string) string {
-		p := NewShTokenizer(dummyLine, str, false)
-		for _, exp := range expected {
-			c.Check(p.ShToken().MkText, equals, exp)
-		}
-		return p.Rest()
-	}
-
-	test := func(str string, expected ...string) {
-		p := NewShTokenizer(dummyLine, str, false)
-		for _, exp := range expected {
-			c.Check(p.ShToken().MkText, equals, exp)
-		}
-		c.Check(p.Rest(), equals, "")
-		t.CheckOutputEmpty()
-	}
-
-	testNil := func(str string) {
-		p := NewShTokenizer(dummyLine, str, false)
-		c.Check(p.ShToken(), check.IsNil)
-		c.Check(p.Rest(), equals, "")
-		t.CheckOutputEmpty()
-	}
-
-	testNil("")
-	testNil(" ")
-	rest := testRest("\t\t\t\n\n\n\n\t ",
-		"\n\n\n\n")
-	c.Check(rest, equals, "\t ")
-
-	test("echo",
-		"echo")
-
-	test("`cat file`",
-		"`cat file`")
-
-	test("PAGES=\"`ls -1 | ${SED} -e 's,3qt$$,3,'`\"",
-		"PAGES=\"`ls -1 | ${SED} -e 's,3qt$$,3,'`\"")
-
-	test("echo hello, world",
-		"echo",
-		"hello,",
-		"world")
-
-	test("if cond1; then action1; elif cond2; then action2; else action3; fi",
-		"if", "cond1", ";", "then",
-		"action1", ";",
-		"elif", "cond2", ";", "then",
-		"action2", ";",
-		"else", "action3", ";",
-		"fi")
-
-	test("PATH=/nonexistent env PATH=${PATH:Q} true",
-		"PATH=/nonexistent",
-		"env",
-		"PATH=${PATH:Q}",
-		"true")
-
-	test("id=$$(${AWK} '{print}' < ${WRKSRC}/idfile)",
-		"id=$$(${AWK} '{print}' < ${WRKSRC}/idfile)")
-
-	test("id=`${AWK} '{print}' < ${WRKSRC}/idfile`",
-		"id=`${AWK} '{print}' < ${WRKSRC}/idfile`")
+	line := t.NewLine("filename.mk", 123, "\ttoken")
+	tok := NewShTokenizer(line, line.Text)
+	t.ExpectPanicMatches(
+		func() { tok.ShAtom(^ShQuoting(0)) },
+		// Normalize the panic message, for Go < 12 if I remember correctly.
+		`^runtime error: index out of range.*`)
 }
 
 func (s *Suite) Test_ShTokenizer_shVarUse(c *check.C) {
+	t := s.Init(c)
 
 	test := func(input string, output *ShAtom, rest string) {
-		tok := NewShTokenizer(nil, input, false)
+		tok := NewShTokenizer(nil, input)
 		actual := tok.shVarUse(shqPlain)
 
-		c.Check(actual, deepEquals, output)
-		c.Check(tok.Rest(), equals, rest)
+		t.CheckDeepEquals(actual, output)
+		t.CheckEquals(tok.Rest(), rest)
 	}
 
 	shvar := func(text, varname string) *ShAtom {
@@ -545,98 +652,78 @@ func (s *Suite) Test_ShTokenizer_shVarUse(c *check.C) {
 	test("$${\\}", nil, "$${\\}")
 }
 
-// This test demonstrates that the shell tokenizer is not perfect yet.
-// There are still some corner cases that trigger a parse error.
-// To get 100% code coverage, they have been found using the fuzzer
-// and trimmed down to minimal examples.
-func (s *Suite) Test_ShTokenizer__examples_from_fuzzing(c *check.C) {
+func (s *Suite) Test_ShTokenizer_ShToken(c *check.C) {
 	t := s.Init(c)
 
-	mklines := t.NewMkLines("fuzzing.mk",
-		MkRcsID,
-		"",
-		"pre-configure:",
-
-		// Covers shAtomBacktDquot: return nil.
-		// These are nested backticks with double quotes,
-		// which should be avoided since POSIX marks them as unspecified.
-		"\t"+"`\"`",
-
-		// Covers shAtomBacktSquot: return nil
-		"\t"+"`'$`",
-
-		// Covers shAtomDquotBacktSquot: return nil
-		"\t"+"\"`'`y",
-
-		// Covers shAtomDquotBackt: return nil
-		// FIXME: Pkglint must parse unescaped dollar in the same way, everywhere.
-		"\t"+"\"`$|",
-
-		// Covers shAtomDquotBacktDquot: return nil
-		// FIXME: Pkglint must support unlimited nesting.
-		"\t"+"\"`\"`",
-
-		// Covers shAtomSubshDquot: return nil
-		"\t"+"$$(\"'",
-
-		// Covers shAtomSubsh: case lexer.AdvanceStr("`")
-		"\t"+"$$(`",
-
-		// Covers shAtomSubshSquot: return nil
-		"\t"+"$$('$)",
-
-		// Covers shAtomDquotBackt: case lexer.AdvanceRegexp("^#[^`]*")
-		"\t"+"\"`# comment")
-
-	mklines.Check()
-
-	// Just good that these redundant error messages don't occur every day.
-	t.CheckOutputLines(
-		"WARN: fuzzing.mk:4: Internal pkglint error in ShTokenizer.ShAtom at \"`\" (quoting=bd).",
-		"WARN: fuzzing.mk:4: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"`\\\"`\"",
-
-		"WARN: fuzzing.mk:5: Internal pkglint error in ShTokenizer.ShAtom at \"$`\" (quoting=bs).",
-		"WARN: fuzzing.mk:5: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"`'$`\"",
-		"WARN: fuzzing.mk:5: Internal pkglint error in MkLine.Tokenize at \"$`\".",
-
-		"WARN: fuzzing.mk:6: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`'`y\"",
-
-		"WARN: fuzzing.mk:7: Internal pkglint error in ShTokenizer.ShAtom at \"$|\" (quoting=db).",
-		"WARN: fuzzing.mk:7: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`$|\"",
-		"WARN: fuzzing.mk:7: Internal pkglint error in MkLine.Tokenize at \"$|\".",
-
-		"WARN: fuzzing.mk:8: Internal pkglint error in ShTokenizer.ShAtom at \"`\" (quoting=dbd).",
-		"WARN: fuzzing.mk:8: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`\\\"`\"",
-
-		"WARN: fuzzing.mk:9: Invoking subshells via $(...) is not portable enough.",
-
-		"WARN: fuzzing.mk:10: Internal pkglint error in ShTokenizer.ShAtom at \"`\" (quoting=S).",
-		"WARN: fuzzing.mk:10: Invoking subshells via $(...) is not portable enough.",
-
-		"WARN: fuzzing.mk:11: Internal pkglint error in ShTokenizer.ShAtom at \"$)\" (quoting=Ss).",
-		"WARN: fuzzing.mk:11: Invoking subshells via $(...) is not portable enough.",
-		"WARN: fuzzing.mk:11: Internal pkglint error in MkLine.Tokenize at \"$)\".",
-
-		"WARN: fuzzing.mk:12: Pkglint ShellLine.CheckShellCommand: splitIntoShellTokens couldn't parse \"\\\"`# comment\"")
-}
-
-// In order to get 100% code coverage for the shell tokenizer, a panic() statement has been
-// added to each uncovered basic block. After that, this fuzzer quickly found relatively
-// small example programs that led to the uncovered code.
-//
-// This test is not useful as-is.
-func (s *Suite) Test_ShTokenizer__fuzzing(c *check.C) {
-	t := s.Init(c)
-
-	fuzzer := NewFuzzer()
-	fuzzer.Char("\"'`$();|_#", 10)
-	fuzzer.Range('a', 'z', 5)
-
-	defer fuzzer.CheckOk()
-	for i := 0; i < 1000; i++ {
-		tokenizer := NewShTokenizer(dummyLine, fuzzer.Generate(50), false)
-		tokenizer.ShAtoms()
-		t.Output() // Discard the output, only react on panics.
+	// testRest ensures that the given string is parsed to the expected
+	// tokens, and returns the remaining text.
+	testRest := func(str string, expected ...string) string {
+		line := t.NewLine("testRest.mk", 1, "")
+		p := NewShTokenizer(line, str)
+		for _, exp := range expected {
+			t.CheckEquals(p.ShToken().MkText, exp)
+		}
+		return p.Rest()
 	}
-	fuzzer.Ok()
+
+	test := func(str string, expected ...string) {
+		line := t.NewLine("test.mk", 1, "")
+		p := NewShTokenizer(line, str)
+		for _, exp := range expected {
+			t.CheckEquals(p.ShToken().MkText, exp)
+		}
+		t.CheckEquals(p.Rest(), "")
+		t.CheckOutputEmpty()
+	}
+
+	testNil := func(str string) {
+		line := t.NewLine("testNil.mk", 1, "")
+		p := NewShTokenizer(line, str)
+		c.Check(p.ShToken(), check.IsNil)
+		t.CheckEquals(p.Rest(), "")
+		t.CheckOutputEmpty()
+	}
+
+	testNil("")
+	testNil(" ")
+	rest := testRest("\t\t\t\n\n\n\n\t ",
+		"\n\n\n\n")
+	t.CheckEquals(rest, "\t ")
+
+	test("echo",
+		"echo")
+
+	test("`cat file`",
+		"`cat file`")
+
+	test("PAGES=\"`ls -1 | ${SED} -e 's,3qt$$,3,'`\"",
+		"PAGES=\"`ls -1 | ${SED} -e 's,3qt$$,3,'`\"")
+
+	test("echo hello, world",
+		"echo",
+		"hello,",
+		"world")
+
+	test("if cond1; then action1; elif cond2; then action2; else action3; fi",
+		"if", "cond1", ";", "then",
+		"action1", ";",
+		"elif", "cond2", ";", "then",
+		"action2", ";",
+		"else", "action3", ";",
+		"fi")
+
+	test("PATH=/nonexistent env PATH=${PATH:Q} true",
+		"PATH=/nonexistent",
+		"env",
+		"PATH=${PATH:Q}",
+		"true")
+
+	test("id=$$(id)",
+		"id=$$(id)")
+
+	test("id=$$(${AWK} '{print}' < ${WRKSRC}/idfile)",
+		"id=$$(${AWK} '{print}' < ${WRKSRC}/idfile)")
+
+	test("id=`${AWK} '{print}' < ${WRKSRC}/idfile`",
+		"id=`${AWK} '{print}' < ${WRKSRC}/idfile`")
 }
